package it.eng.spago.workflow.shark.impl;

import it.eng.spago.workflow.api.IWorkflowAssignment;
import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowPackage;
import it.eng.spago.workflow.api.IWorkflowProcess;
import it.eng.spago.workflow.api.WorkflowEngineException;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmodel.WfAssignment;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfmodel.WfResource;
import org.enhydra.shark.api.client.wfservice.ExecutionAdministration;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WfProcessMgrIterator;

 
/**
 * @author zoppello
 * The implementation of IWorkflowConnection for shark
 */
public class SharkWorflowConnectionImpl implements IWorkflowConnection{
	
	/**
	 * the shark specific connection
	 */
	private SharkConnection aSharkConnection = null;
	/**
	 * The user id
	 */
	private String userId = null;
	/**
	 * The password used to connect
	 */
	private String pwd = null;
	
	/**
	 * 
	 */
	private boolean isConnected = false;
	
	/**
	 * Constructor. It takes the shark connection
	 * @param pSharkConnection
	 */
	public SharkWorflowConnectionImpl(SharkConnection pSharkConnection){
		this.aSharkConnection = pSharkConnection;
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#close()
	 */
	public void close() throws WorkflowEngineException {
		try{
			this.aSharkConnection.disconnect();
			this.userId = null;
			this.pwd = null;
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#getAssignments()
	 */
	public List getAssignments() throws WorkflowEngineException {
		try{
			System.out.println("Get Activities");
			WfResource wfResource =  aSharkConnection.getResourceObject();
		
			System.out.println(" Founded Activities ["+ wfResource.how_many_work_item() + "]");
			
			List assigments = new ArrayList();
			
			WfAssignment[] allWfAssignment = wfResource.get_sequence_work_item(0);
			//WfProcess wfProcess = null;
			//WfActivity wfActivity = null;
			//WfAssignment wfAssignment = null;
			
			
			for (int i = 0; i < allWfAssignment.length; ++i) {
				/*
				wfAssignment = allWfAssignment[i];
				wfActivity = wfAssignment.activity();
				wfProcess = wfActivity.container();
				*/
				assigments.add(new SharkWorkflowAssignmentImpl(allWfAssignment[i]));
			}
			return assigments;
				/*
				wfResource = wfAssignment.assignee();
				
				System.out.println(" WfResource key ["+wfResource.resource_key()+"]");
				System.out.println(" WfResource name ["+wfResource.resource_name()+"]");
				
				
				System.out.println(" Process Key ["+ wfProcess.key() + "]");
				System.out.println(" Process Description ["+ wfProcess.description() + "]");
				System.out.println(" Process Name ["+ wfProcess.name() + "]");
				System.out.println(" Process State ["+ wfProcess.state() + "]");
								
				
				System.out.println(" Activity Key [" + wfActivity.key() + "]");
				System.out.println(" Activity Key [" + wfActivity.name() + "]");
				System.out.println(" Activity Description [" + wfActivity.description() + "]");
				
				System.out.println(" Activity priority [" + wfActivity.priority() + "]");
				System.out.println(" Activity state [" + wfActivity.state() + "]");	

				System.out.println(" ----------------- Context ---------------------------------- ");
				Map processContext = wfActivity.process_context();
				
				Iterator it = processContext.keySet().iterator();
				Object ctxKey = null;
				while (it.hasNext()){
					ctxKey = it.next();
					System.out.println( "["+ctxKey+"] ["+processContext.get(ctxKey)+"]");
				}
				
			}
			*/
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
		
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#createProcess(java.lang.String, java.lang.String)
	 */
	public IWorkflowProcess createProcess(String workflowPackageID, String workflowID) throws WorkflowEngineException {
		try {
			return new SharkWorkflowProcessImpl(aSharkConnection.createProcess(workflowPackageID, workflowID));
			
		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}
	
	

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#open(java.lang.String, java.lang.String)
	 */
	public void open(String userId, String userPassword) throws WorkflowEngineException {
		try{
			aSharkConnection.connect(userId,userPassword,"","");
			this.userId = userId;
			this.pwd = userPassword;
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#createProcessWithInitialContext(java.lang.String, java.lang.String, java.util.Map)
	 */
	public IWorkflowProcess createProcessWithInitialContext(String workflowPackageID, String workflowID, Map context) throws WorkflowEngineException {
		IWorkflowProcess wfProcess = createProcess(workflowPackageID, workflowID);
		wfProcess.setContext(context);
		return wfProcess;
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#getWorkflowAssignment(java.lang.String)
	 */
	public IWorkflowAssignment getWorkflowAssignment(String activityId) throws WorkflowEngineException {
		try{
			WfAssignment a = null;
			WfAssignment[] ar = aSharkConnection.getResourceObject().get_sequence_work_item(0);
			for (int i = 0; i < ar.length; ++i) {
				if (activityId.equals(ar[i].activity().key())) {
					a = ar[i];
					break;
				}
			}
			return new SharkWorkflowAssignmentImpl(a);
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#getProcessForPackage(it.eng.spago.workflow.api.IWorkflowPackage)
	 */
	public List getProcessForPackage(IWorkflowPackage pkg) throws WorkflowEngineException {
		try{
			ExecutionAdministration execAdmin = Shark.getInstance().getAdminInterface().getExecutionAdministration();
			
			execAdmin.connect(userId, pwd, "", "");
			WfProcessMgrIterator pmi=execAdmin.get_iterator_processmgr (); 
			String query="packageId.equals(\""+pkg.getPackageID()+"\")";
			pmi.set_query_expression(query);
			WfProcessMgr[] mgrs=pmi.get_next_n_sequence(0);
			
			List result = new ArrayList();
			WfProcess[] activeProcess = null; 
			
			for (int i=0; i < mgrs.length; i++){
				activeProcess = mgrs[i].get_iterator_process().get_next_n_sequence(0);
				for (int j=0; j < activeProcess.length; j++){
					result.add(new SharkWorkflowProcessImpl(activeProcess[j]));
					
				}
			}
			execAdmin.disconnect();
			return result;
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}
	
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#deleteProcess(it.eng.spago.workflow.api.IWorkflowProcess)
	 */
	public void deleteProcess(IWorkflowProcess process) throws WorkflowEngineException {
		try{
			
			ExecutionAdministration execAdmin = Shark.getInstance().getAdminInterface().getExecutionAdministration();
			
			execAdmin.connect(userId, pwd, "", "");
			execAdmin.deleteClosedProcess(process.getId());
			
			execAdmin.disconnect();
		
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#getAllWorkflowProcessDefinition()
	 */
	public  List getAllWorkflowProcessDefinition() throws WorkflowEngineException {
		try{
			ExecutionAdministration execAdmin = Shark.getInstance().getAdminInterface().getExecutionAdministration();
			
			execAdmin.connect(userId, pwd, "", "");
			WfProcessMgrIterator pmi=execAdmin.get_iterator_processmgr (); 
			WfProcessMgr[] mgrs=pmi.get_next_n_sequence(0);
			
			List result = new ArrayList();
		
			for (int i=0; i < mgrs.length; i++){
				result.add(new SharkWorkflowProcessDefinition(mgrs[i]));
			}
			execAdmin.disconnect();
			return result;
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowConnection#getWorkflowProcessDefinition(it.eng.spago.workflow.api.IWorkflowPackage)
	 */
	public List getWorkflowProcessDefinition(IWorkflowPackage pkg) throws WorkflowEngineException {
		try{
			ExecutionAdministration execAdmin = Shark.getInstance().getAdminInterface().getExecutionAdministration();
			
			execAdmin.connect(userId, pwd, "", "");
			WfProcessMgrIterator pmi=execAdmin.get_iterator_processmgr (); 
			String query="packageId.equals(\""+pkg.getPackageID()+"\")";
			pmi.set_query_expression(query);
			WfProcessMgr[] mgrs=pmi.get_next_n_sequence(0);
			
			List result = new ArrayList();
			
			for (int i=0; i < mgrs.length; i++){
				result.add(new SharkWorkflowProcessDefinition(mgrs[i]));
			}
			
			execAdmin.disconnect();
			
			return result;
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}
	
	
}
