package it.eng.spago.workflow.shark.impl;

import it.eng.spago.workflow.api.IWorkflowAssignment;
import it.eng.spago.workflow.api.WorkflowEngineException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmodel.WfAssignment;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.common.SharkConstants;

/**
 * @author zoppello
 * This is the implementation of IWorkflowAssignment for Shark Engine
 */
public class SharkWorkflowAssignmentImpl implements IWorkflowAssignment {

	/**
	 * Shark assignment internal object
	 */
	private WfAssignment aWfAssignment = null;
	/**
	 * The map of the extend attributes for the activity
	 */
	private Map activityExtendedAttributes = null;
	
	/**
	 * The name of the extended attribute that conatin the eventually mapped form
	 */
	private static final String EXTENDED_ATTRIBUTE_NAME_FOR_MAPPED_TASK = "form";
	
	/**
	 * 
	 */
	private void initExtendedAttributes(){
		
		AdminMisc am=Shark.getInstance().getAdminInterface().getAdminMisc();

		activityExtendedAttributes = new HashMap();
		try{
			String[][] extAttrs=am.getActivitiesExtendedAttributeNameValuePairs(aWfAssignment.activity().container().key(),aWfAssignment.activity().key());
			for (int i=0; i < extAttrs.length; i++){
				activityExtendedAttributes.put(extAttrs[i][0],extAttrs[i][1]);
			}
		}catch(Exception e ){
			e.printStackTrace();
		}
	}
	/**
	 * Constructor. It takes the Shark assignment object
	 * @param aWfAssignment
	 */
	public SharkWorkflowAssignmentImpl(WfAssignment aWfAssignment){
		this.aWfAssignment = aWfAssignment;
		initExtendedAttributes();
	}
	
	/** 
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getProcessKey()
	 */
	public String getProcessKey() throws WorkflowEngineException{
		try{
			return aWfAssignment.activity().container().key();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getProcessName()
	 */
	public String getProcessName() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().container().name();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getProcessDescription()
	 */
	public String getProcessDescription() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().container().description();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getProcessState()
	 */
	public String getProcessState() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().container().state();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getActivityKey()
	 */
	public String getActivityKey() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().key();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getActivityName()
	 */
	public String getActivityName() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().name();
			
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getActivityDescription()
	 */
	public String getActivityDescription() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().description();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getActivityPriority()
	 */
	public short getActivityPriority() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().priority();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getActivityState()
	 */
	public String getActivityState() throws WorkflowEngineException {
		try{
			return aWfAssignment.activity().state();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#isAccepted()
	 */
	public boolean isAccepted() throws WorkflowEngineException {
		try{
			return aWfAssignment.get_accepted_status();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#complete()
	 */
	public void complete() throws WorkflowEngineException {
		try{
			 aWfAssignment.activity().complete();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#abort()
	 */
	public void abort() throws WorkflowEngineException {
		try{
			 aWfAssignment.activity().abort();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getContext()
	 */
	public Map getContext() throws WorkflowEngineException {
		try{
			 return aWfAssignment.activity().process_context();
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#setContext(java.util.Map)
	 */
	public void setContext(Map context) throws WorkflowEngineException {
		try{
			aWfAssignment.activity().set_result(context);
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		
		try{
			StringBuffer aStringBuffer = new StringBuffer();
			aStringBuffer.append(" ============================ Assignment =======================================\n");
			aStringBuffer.append("  --> Resource Key ["+aWfAssignment.assignee().resource_key()+"] Name ["+aWfAssignment.assignee().resource_name()+"]\n" );
			aStringBuffer.append("  --> Process      ["+aWfAssignment.activity().container().key()+"] Name ["+aWfAssignment.activity().container().name()+"] Description ["+aWfAssignment.activity().container().description()+"] Status ["+aWfAssignment.activity().container().state()+"]\n");
			aStringBuffer.append("  --> Activity Key ["+aWfAssignment.activity().key()+"] Name ["+aWfAssignment.activity().name()+"] Description ["+aWfAssignment.activity().description()+"] Status ["+aWfAssignment.activity().state()+"]\n");
			aStringBuffer.append("  --> Assignment Status ["+ aWfAssignment.get_accepted_status()+ "]\n");
			
			aStringBuffer.append("  --> Extended Attribute <--\n");
			Iterator it = this.activityExtendedAttributes.keySet().iterator();
			Object keyExtAttribute = null;
			while (it.hasNext()){
				keyExtAttribute = it.next();
				aStringBuffer.append("   ACTIVITY EXT ATTRIBUTE [" + keyExtAttribute + "] ["+this.activityExtendedAttributes.get(keyExtAttribute)+"]\n");
			}
			aStringBuffer.append("  --> Process Context <--\n");
			
			Map processContext  = getContext();
			
			it = processContext.keySet().iterator();
		
			Object key = null;
			while (it.hasNext()){
				key = it.next();
				aStringBuffer.append(" PROCESS VARIABLE   [" + key + "] ["+processContext.get(key)+"]\n");
			}
			aStringBuffer.append(" ======================== End Assignment ========================================\n");
			return aStringBuffer.toString();
		}catch(Exception e){
			e.printStackTrace();
			return new String(" Error in toString()");
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#accept()
	 */
	public void accept() throws WorkflowEngineException {
		try{
			if (!isAccepted()) {
				aWfAssignment.activity().change_state(SharkConstants.STATE_OPEN_RUNNING);
			}
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
		
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#reject()
	 */
	public void reject() throws WorkflowEngineException {
		try{
			if (isAccepted()) {
				aWfAssignment.activity().change_state(SharkConstants.STATE_OPEN_NOT_RUNNING_NOT_STARTED);
				
			}
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
		
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowAssignment#getMappedForm()
	 */
	public String getMappedForm()  {
		String mappedTask = null;
		if ((this.activityExtendedAttributes != null)){
			mappedTask =  (String)this.activityExtendedAttributes.get(EXTENDED_ATTRIBUTE_NAME_FOR_MAPPED_TASK);
		}
		return mappedTask;
	}



}
