package it.eng.spago.workflow.shark.impl;

import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowEngine;
import it.eng.spago.workflow.api.IWorkflowPackage;
import it.eng.spago.workflow.api.WorkflowEngineException;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfservice.PackageAdministration;
import org.enhydra.shark.api.client.wfservice.RepositoryMgr;
import org.enhydra.shark.api.client.wfservice.SharkConnection;




/**
 * @author zoppello
 * 
 * The implementation of IWorkflowEngine for Shark
 */
public class SharkWorkflowEngineImpl implements IWorkflowEngine {
	

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#getWorkflowConnection()
	 */
	public IWorkflowConnection getWorkflowConnection() throws WorkflowEngineException {
		try{
			SharkConnection aSharkConnection = Shark.getInstance().getSharkConnection();
			
			return new SharkWorflowConnectionImpl(aSharkConnection);
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#loadPackageDefinition(java.lang.String)
	 */
	public IWorkflowPackage loadPackageDefinition(String worflowDefinitionFileName) throws WorkflowEngineException{
		try{
		PackageAdministration pa = Shark.getInstance()
		.getAdminInterface()
		.getPackageAdministration();
		
		String[] openedPackages = pa.getOpenedPackageIds();
		
		for (int i=0; i < openedPackages.length; i++){
			System.out.println(" -- Opened Package [" + openedPackages[i] + "]");
		}
		
		String pkgId = Shark.getInstance()
			.getRepositoryManager()
			.getPackageId(worflowDefinitionFileName);
		
		
		if (!pa.isPackageOpened(pkgId)) {
			pa.openPackage(worflowDefinitionFileName);
		}
		
		pa.refreshXPDLCache();
		pa.synchronizeXPDLCache();
		
		return new SharkWorkflowPackageImpl(pkgId, pa
				.getCurrentPackageVersion(pkgId), worflowDefinitionFileName);
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
		
	}

	

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#unloadPackageDefinition(it.eng.spago.workflow.api.IWorkflowPackage)
	 */
	public void unloadPackageDefinition(IWorkflowPackage pkg) throws WorkflowEngineException {
		try{
		PackageAdministration pa = Shark.getInstance()
		.getAdminInterface()
		.getPackageAdministration();
	
		
	
		if (pa.isPackageOpened(pkg.getPackageID())) {
			System.out.println(" Package [" + pkg.getPackageID() + "] is Open now we close IT!!" );
			pa.closePackage(pkg.getPackageID());	
		}else{
			System.out.println(" Package [" + pkg.getPackageID() + "] is Already Closed" );
		}
		
		pa.refreshXPDLCache();
		pa.synchronizeXPDLCache();
		
		System.out.print(" -- After Closing --");
		String[] openedPackages = pa.getOpenedPackageIds();
		
		for (int i=0; i < openedPackages.length; i++){
			System.out.print(" -- Opened Package [" + openedPackages[i] + "]");
		}
		}catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
		
	}



	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#getLoadedPackges()
	 */
	public List getLoadedPackges() throws WorkflowEngineException {
		try {
			RepositoryMgr repositoryManager = Shark.getInstance().getRepositoryManager();
			
			PackageAdministration pa  = Shark.getInstance().getAdminInterface().getPackageAdministration();
			
			String[] packagePaths = repositoryManager.getPackagePaths();
			
			String pkgPath = null;
			String pkgId = null;
			List result = new ArrayList();
			for (int i=0; i < packagePaths.length; i++){
				pkgPath = packagePaths[i];
				pkgId = repositoryManager.getPackageId(pkgPath); 
				if (pa.isPackageOpened(pkgId))
					result.add(new SharkWorkflowPackageImpl(pkgId, pa.getCurrentPackageVersion(pkgId), pkgPath));
			}
			
			
			return result;
		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
		
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#getWorkflowPackage(java.lang.String)
	 */
	public IWorkflowPackage getWorkflowPackage(String pkgId)
			throws WorkflowEngineException {
		try {
			PackageAdministration pa = Shark.getInstance().getAdminInterface()
					.getPackageAdministration();

			return new SharkWorkflowPackageImpl(pkgId, pa
					.getCurrentPackageVersion(pkgId), getPackagePathById(pkgId));

		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}

	}
	
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#getPackagePathById(java.lang.String)
	 */
	public String getPackagePathById(String pkgId)
			throws WorkflowEngineException {
		try {
			RepositoryMgr repositoryManager = Shark.getInstance()
					.getRepositoryManager();

			String paths[] = repositoryManager.getPackagePaths();

			Map pathToIdMap = repositoryManager.getPackagePathToIdMapping();
			String tmpId = null;
			for (int i = 0; i < paths.length; i++) {
				tmpId = (String) pathToIdMap.get(paths[i]);
				if (tmpId.equals(pkgId)) {
					return paths[i];
				}
			}
			return null;
		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowEngine#getAvailablePackages()
	 */
	public List getAvailablePackages() throws WorkflowEngineException {
		try {
			RepositoryMgr repositoryManager = Shark.getInstance().getRepositoryManager();
			
			
			
			String[] packagePaths = repositoryManager.getPackagePaths();
			
			String pkgPath = null;
			String pkgId = null;
			List result = new ArrayList();
			for (int i=0; i < packagePaths.length; i++){
				pkgPath = packagePaths[i];
				pkgId = repositoryManager.getPackageId(pkgPath); 
				result.add(new SharkWorkflowPackageImpl(pkgId, "-1", pkgPath));
			}
			return result;
		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
		
	
	}





}
