package it.eng.spago.workflow.shark.impl;

import it.eng.spago.workflow.api.IWorkflowPackage;
import it.eng.spago.workflow.api.WorkflowEngineException;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfservice.PackageAdministration;

/**
 * @author zoppello
 *
 * This class is the implementation of IWorkflowPackage for Shark workflow Engine
 */
public class SharkWorkflowPackageImpl implements IWorkflowPackage{
	
	/**
	 * The package identifier
	 */
	private String packageID = null;
	/**
	 * The package version
	 */
	private String packageVersion = null;
	
	/**
	 * The file name with package definition 
	 */
	private String packageDefinitionFileName = null;
	
	/**
	 * Constructor it takes in input the package identifier, the package version, the fileName tha define package
	 * @param packageid the package identifier
	 * @param version the package version
	 * @param packageDefinitionFileName the fileName tha define package
	 */
	public SharkWorkflowPackageImpl(String packageid, String version, String packageDefinitionFileName) {
		super();
		this.packageID = packageid;
		this.packageVersion = version;
		this.packageDefinitionFileName = packageDefinitionFileName;
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowPackage#getPackageID()
	 */
	public String getPackageID() {
		return packageID;
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowPackage#getPackageVersion()
	 */
	public String getPackageVersion() {
		return packageVersion;
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowPackage#getPackageDefinitionFileName()
	 */
	public String getPackageDefinitionFileName() {
		return packageDefinitionFileName;	
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowPackage#isReferenced()
	 */
	public boolean isReferenced() throws WorkflowEngineException {
		
		try{
			PackageAdministration pa = Shark.getInstance()
			.getAdminInterface()
			.getPackageAdministration();
			
			return  pa.isPackageOpened(this.packageID);
			
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	
	
}
