package it.eng.spago.workflow.shark.impl;

import it.eng.spago.workflow.api.IWorkflowProcess;
import it.eng.spago.workflow.api.WorkflowEngineException;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.enhydra.shark.api.client.wfmodel.WfProcess;

/**
 * @author zoppello
 *
 * This class is the implementation of IWorkflowProcess for Shark workflow Engine
 */
public class SharkWorkflowProcessImpl implements IWorkflowProcess {
	
	/**
	 * The object of sharl that reperesent a process
	 */
	private WfProcess aWfProcess = null;
	
	/**
	 * Constructor. The SharkWorkflowProcessImpl need the internal shark object
	 * @param aWfProcess the process in shark
	 */
	public SharkWorkflowProcessImpl(WfProcess aWfProcess ){
		this.aWfProcess = aWfProcess;
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#getId()
	 */
	public String getId() throws WorkflowEngineException {
		try{
			return aWfProcess.key();
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#getName()
	 */
	public String getName() throws WorkflowEngineException {
		try{
			return aWfProcess.name();
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#getDescription()
	 */
	public String getDescription() throws WorkflowEngineException {
		try{
			return aWfProcess.description();
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}
	
	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#start()
	 */
	public void start() throws WorkflowEngineException{
		try{
			aWfProcess.start();
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#abort()
	 */
	public void abort() throws WorkflowEngineException {
		try{
			aWfProcess.abort();
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
		
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#getContext()
	 */
	public Map getContext() throws WorkflowEngineException {
		try {
			return aWfProcess.process_context();
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#setContext(java.util.Map)
	 */
	public void setContext(Map pContext) throws WorkflowEngineException {
		try{
		Map processContext = aWfProcess.process_context();
	    Set allKeys = pContext.keySet();
	    Iterator iterator = allKeys.iterator();
	    while (iterator.hasNext()) {
	        String nextVariable = (String) iterator.next();
	        if (processContext.containsKey(nextVariable)){
				processContext.put(nextVariable, pContext.get(nextVariable));
				aWfProcess.set_process_context(processContext);
			}
			else {
				System.out.println("variable " + nextVariable + " not found as a variable for this process");
			}
	    }
		}catch(Exception e){
			throw new WorkflowEngineException(e);
		}
		
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#isClosed()
	 */
	public boolean isClosed() throws WorkflowEngineException {
		
		try {
			return aWfProcess.state().startsWith("closed");
		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	/**
	 * @see it.eng.spago.workflow.api.IWorkflowProcess#getState()
	 */
	public String getState() throws WorkflowEngineException {

		try {
			return aWfProcess.state();
		} catch (Exception e) {
			throw new WorkflowEngineException(e);
		}
	}

	
		
	
}
