package it.eng.spago.workflow.api;

import java.util.List;

/**
 * @author zoppello
 * This is the core interface of SpagoWorkflowAPI that represent a Worflow Engine
 * 
 * In Spago Worflow API The workflow Engine is responsible for
 * 
 * <ul>
 * <li> Getting a connection to workflow</li>
 * <li> Load and unload packages in the engine ( Packages are resources that countain workflow definition )</li>
 * <li> Get the list of the packages loaded and the list of the packages that can be loaded </li>
 * <li> Get a package instance given the pakage id </li>
 * </ul>
 */
public interface IWorkflowEngine {
	
	/**
	 * This method get a connection to the engine
	 * @return a IWorkflowConnection object to make operation on the engine
	 * @throws WorkflowEngineException if some error occurs getting connection by the engine
	 */
	public IWorkflowConnection getWorkflowConnection() throws WorkflowEngineException;
	
	/**
	 * This method load a packege  containing Workflow Definitions in the engine
	 * @param pkgDefinitionFileName The file containing the package definition
	 * @return a IWorkflowPackage a IWorkflow Package object representig the package loaded in the engine
	 * @throws WorkflowEngineException if some error occurs during the loading of the packege in the engine
	 */
	public IWorkflowPackage loadPackageDefinition(String pkgDefinitionFileName) throws WorkflowEngineException;
	/**
	 * This method cause the unload of the IWorkflow package by the engine
	 * After this operation we cannot use anymore the IWorkflowPackage until a loadPackage Occurs
	 * @param pkg IWorkflowPackage that must be unloaded by the engine
	 * @throws WorkflowEngineException
	 */
	public void unloadPackageDefinition(IWorkflowPackage pkg) throws WorkflowEngineException;
	
	/**
	 * This method retrieve the List of the packages that are loaded in the engine
	 * @return the list of IWorkflowPackage object loaded in the engine
	 * @throws WorkflowEngineException
	 */
	public List getLoadedPackges() throws WorkflowEngineException;
	
	/**
	 * This method retrieve the List of the packages that can be loaded in the engine but that are not loaded
	 * in the engine
	 * @return the list of the packages available but that are not laoded in the engine
	 * @throws WorkflowEngineException
	 */
	public List getAvailablePackages() throws WorkflowEngineException;

	/**
	 * This method return an instance of IWorkflowPackage given a package id
	 * @param pkgId The id of the package to retrieve
	 * @return The IWorkflow package instance 
	 * @throws WorkflowEngineException if some error occurs getting a package
	 */
	public IWorkflowPackage getWorkflowPackage(String pkgId) throws WorkflowEngineException;

	/**
	 * Return the package phisical path given the path id
	 * @param pkgId The package Identifier for wivh we want to retrieve the path
	 * @return hisical path given the path id
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public String getPackagePathById(String pkgId) throws WorkflowEngineException;
	
}
