package it.eng.spago.workflow.api;

import java.util.Map;

/**
 * @author zoppello
 *
 * IWorkflow Process is tghe interface for the run time instances of the business process in the
 * engine.
 * 
 * It's very important not confuse IWorkflowProcess with IWorkflowDefinition objects.
 * 
 * For a IWorkflowDefintion object we can have multiple instance of IWorkflowProcess Objects.
 * 
 * This class has all the method to manage a run time process and it's context.
 * 
 * Each instance of process has a map of variables called context, a runtime state a name and a 
 * description.
 */
public interface IWorkflowProcess {

	/**
	 * This method start the process. It's not enough to get an instance of the IWorkflowProcess
	 * to start the process, we must call this method to start the execution of the process
	 * @throws WorkflowEngineException if some errors occurs.
	 */
	public void start() throws WorkflowEngineException;
	
	/**
	 * This method abort the process execution before the end of execution.
	 * @throws WorkflowEngineException if some errors occurs.
	 */
	public void abort() throws WorkflowEngineException;
	
	/**
	 * This method return the context of the process
	 * @return the context ( The map of the variable of the process )
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public Map getContext() throws WorkflowEngineException;
	
	/**
	 * This method update the context ot the process
	 * @param context the new context for this process
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public void setContext(Map context) throws WorkflowEngineException;
	
	/**
	 * This metho eturn the identifier of the process in the engine
	 * @return the identifier of the process in the engines
	 * @throws WorkflowEngineException
	 */
	public String getId() throws WorkflowEngineException;
	
	/**
	 * This method verify if a process is terminated
	 * @return true if the process is closed ( terminated )
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public boolean isClosed() throws WorkflowEngineException;
	
	/**
	 * Get the current state of the process
	 * @return the state of the project
	 * @throws WorkflowEngineExceptionif some errors occurs
	 */
	public String getState() throws WorkflowEngineException;
	
	/**
	 * Get the name of the process
	 * @return The name of the process
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public String getName() throws WorkflowEngineException;
	
	/**
	 * This method return the process description
	 * @return the descritpion of the process
	 * @throws WorkflowEngineException if some errors occurs
	 */
	public String getDescription() throws WorkflowEngineException;

}
