package it.eng.spago.workflow.worklist;

import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowPackage;
import it.eng.spago.workflow.api.IWorkflowProcessDefinition;

import java.util.Iterator;
import java.util.List;


/**
 * @author zoppello
 *
 * An utility Class to generate Javascript code to generate trees in JSP Page
 * This class generate code for dTree javascript library
 */
public class Javascript {

	/**
	 * This method generate the Javascript code to render a Tree in Web pages given a IWorkflowConnection 
	 * and a list of packages to render
	 * @param wfConn - A Workflow Connection Object (See SpagoWorkflowAPI)
	 * @param packages - The list of the packages to be included in Javascript Tree
	 * @return Generate the Javascript code to render trees in Web pages
	 */
	public static String createPackagesTree(IWorkflowConnection wfConn,List packages){
		return createPackagesTree(wfConn, packages, "img");
	}
	/**
	 * This method generate the Javascript code to render a Tree in Web pages given a IWorkflowConnection 
	 * and a list of packages to render, It renders the packages and the process definitions for each package
	 * @param wfConn - A Workflow Connection Object (See SpagoWorkflowAPI)
	 * @param packages - The list of the packages to render
	 * @param imgFolder - The path used to get the images associated to the tree nodes
	 * @return
	 */
	public static String createPackagesTree(IWorkflowConnection wfConn, List packages, String imgFolder){
		StringBuffer aStringBuffer  = new StringBuffer();
		
		String nomeAlberoJavaScript = "repTree";
		aStringBuffer.append("<script type=\"text/javascript\">");
		aStringBuffer.append(nomeAlberoJavaScript +"= new dTree('"+nomeAlberoJavaScript+"');");
		aStringBuffer.append(nomeAlberoJavaScript +".add(0,-1,'Workflow System','','','Workflow System','../"+imgFolder+"/base.gif','../"+imgFolder+"/base.gif');");
		
		
		
		
		
		String baseProcessOfPackageURL ="../servlet/AdapterHTTP?PAGE=ProcessMonitorPage&MODULE=ProcessForPackageModule&PackageID=";
		List processDefinitionForPackage = null;
		if (packages != null){
			IWorkflowPackage pkg = null;
			int nodeCounter = 0;
			int rootNode = 0;
			String label = null;
			String tmpUrl = null;
			String procUrl = null;
			for (Iterator it = packages.iterator(); it.hasNext();){
				pkg = (IWorkflowPackage)it.next();
				label = pkg.getPackageID();
				nodeCounter++; 
				tmpUrl = baseProcessOfPackageURL + pkg.getPackageID();
				aStringBuffer.append(nomeAlberoJavaScript+".add("+nodeCounter+","+rootNode+",'"+label+"','"+tmpUrl+"','"+label+"','_self','../"+imgFolder+"/Package.gif','../"+imgFolder+"/Package.gif');");
				try{
					processDefinitionForPackage = wfConn.getWorkflowProcessDefinition(pkg);
				}catch(Exception e){
					e.printStackTrace();
				}
				int parentNode = nodeCounter;
				if (processDefinitionForPackage != null){
					IWorkflowProcessDefinition procDef = null;
					for (Iterator itProcess = processDefinitionForPackage.iterator(); itProcess.hasNext();){
						procDef = (IWorkflowProcessDefinition)itProcess.next();
						
						procUrl = "#";
						try{
							procUrl = tmpUrl+"&ProcessDefinitionName="+procDef.getProcessDefinitionId();
						}catch(Exception e){
							e.printStackTrace();
						}
						
						nodeCounter++; 
						label = "Generic Process";
						try{
							label = procDef.getProcessDefinitionId();
						}catch(Exception e){
							e.printStackTrace();
						}
						
						aStringBuffer.append(nomeAlberoJavaScript+".add("+nodeCounter+","+parentNode+",'"+label+"','"+procUrl+"','"+label+"','_self','../"+imgFolder+"/ProcessDef.gif','../"+imgFolder+"/ProcessDef.gif');");
					}//for (Iterator itProcess = processDefinitionForPackage.iterator(); itProcess.hasNext();){
					
				}//if (processDefinitionForPackage != null){
			}//for (Iterator it = packages.iterator(); it.hasNext();){
		}
		aStringBuffer.append("document.write("+nomeAlberoJavaScript+");");
		aStringBuffer.append("</script>");
		return aStringBuffer.toString();
	}
	
	
}
