
package it.eng.spago.workflow.worklist.action;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.workflow.api.IWorkflowAssignment;
import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowEngine;





/**
 * @author Zoppello
 *
 * A Spago Action  that use SpagoWorkflowAPI to make the user in current session accept an
 * assignment
 */
public class AcceptActivityAction extends AbstractAction{

    /** 
     * @see it.eng.spago.dispatching.service.ServiceIFace#service(it.eng.spago.base.SourceBean, it.eng.spago.base.SourceBean)
     */
    public void service(SourceBean aRequest, SourceBean aResponse) throws Exception {
       
    	// Getting Containers
    	SessionContainer session = getRequestContainer().getSessionContainer();
    	ApplicationContainer application = ApplicationContainer.getInstance();
    	IEngUserProfile userProfile = (IEngUserProfile)session.getAttribute(IEngUserProfile.ENG_USER_PROFILE);
    	
    	//Getting Workflow Engine 
    	IWorkflowEngine wfEngine = (IWorkflowEngine)application.getAttribute("WfEngine");
    	
    	
    	// Getting Connection From Workflow
    	IWorkflowConnection wfConnection = wfEngine.getWorkflowConnection();
    	
    	
    	// Open the connection
    	wfConnection.open((String)userProfile.getUserUniqueIdentifier(), (String)userProfile.getUserAttribute("password"));
    	
    	
    	//Get From the Reuest the activity to accept
    	String activityKey = (String)aRequest.getAttribute("ActivityKey");
    	
    	
    	//Get the assignment relative to activity
    	IWorkflowAssignment wfAssignment = wfConnection.getWorkflowAssignment(activityKey);
    	
    	
    	//Accept Actitity
    	wfAssignment.accept();
    	
    	//See if the activity is mapped on a form
    	String mappedForm = wfAssignment.getMappedForm();
    	
    	
    	
    	if (mappedForm == null){
    		// If Activity is not mapped to a particular form it can complete
    		wfAssignment.complete();
    	}else {
    		// Setting the target form and let decide the AcceptActivityPublisher
    		aResponse.setAttribute("TargetForm", mappedForm);
    	}
    	
    	//Close Workflow Connection
    	wfConnection.close();
    	
    }
}
