
package it.eng.spago.workflow.worklist.action;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.mappers.SourceBeanToProcessContextMapper;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.workflow.api.IMapper;
import it.eng.spago.workflow.api.IWorkflowAssignment;
import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowEngine;

import java.util.Iterator;
import java.util.Map;





/**
 * @author Zoppello
 * 
 * A Spago Action  that use SpagoWorkflowAPI to make the user in current session complete or reject an
 * assignment that has already accepted
 */
public class CompleteRejectActivityAction extends AbstractAction{

    /** 
     * @see it.eng.spago.dispatching.service.ServiceIFace#service(it.eng.spago.base.SourceBean, it.eng.spago.base.SourceBean)
     */
    public void service(SourceBean aRequest, SourceBean aResponse) throws Exception {
    	
    	// This action handle both activity completion and activity reject 
    	
    	//Getting Containers
    	SessionContainer session = getRequestContainer().getSessionContainer();
    	ApplicationContainer application = ApplicationContainer.getInstance();
    	IEngUserProfile userProfile = (IEngUserProfile)session.getAttribute(IEngUserProfile.ENG_USER_PROFILE);
    	
    	//Getting Workflow Engine 
    	IWorkflowEngine wfEngine = (IWorkflowEngine)application.getAttribute("WfEngine");
    	
    	
    	
    	//Getting Connection From Workflow
    	IWorkflowConnection wfConnection = wfEngine.getWorkflowConnection();
    	
    	
    	// Open the connection
    	wfConnection.open((String)userProfile.getUserUniqueIdentifier(), (String)userProfile.getUserAttribute("password"));
    	
    	//Gettin Activity Key 
    	String activityKey = (String)aRequest.getAttribute("ActivityKey");
    	
    	// Gettin workflow assignment relative to key
    	IWorkflowAssignment wfAssignment = wfConnection.getWorkflowAssignment(activityKey);
    	
    	
    	
    	if (aRequest.getAttribute("CompleteActivity") != null){
    		// Submit buttin named CompleteActivity is pressed
    		
    		System.out.println("Completing Activity ["+ activityKey + "]");
    		
    		// Getting activity Process Context
    		Map context = wfAssignment.getContext(); 
    		
    		IMapper mapper = new SourceBeanToProcessContextMapper();
    		
    		Map newContex = mapper.getContext(aRequest);
    		
    		Iterator it = newContex.keySet().iterator();	
    		Object key = null;
    		Object value = null;
    		while (it.hasNext()){
    			key = it.next();
    			value = newContex.get(key);
    			if (context.keySet().contains(key)){
    				
    				System.out.println(" Updating the value of ["+key+"] with value ["+value+"]" );
    				context.put(key, value);
    			}else{
    				System.out.println(" The process does not contain a variable called ["+key+"]" );
    			}
    		}
    		wfAssignment.setContext(context);
    		// Complete Activity
    		wfAssignment.complete();
    		
    		
    	}else{
    		//  Submit buttin named RejectActivity is pressed
    		System.out.println("Rejecting Activity ["+ activityKey + "]");
        	wfAssignment.reject();
    	}
    	
    	// PUBLISH TO WORKLIST
    	aRequest.setAttribute("PUBLISHER_NAME", "LOOP_LISTA_FLUSSI_PUBLISHER");
    	
    	// Close Connection
    	wfConnection.close();
    	
    	
    }
}
