
package it.eng.spago.workflow.worklist.action;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dispatching.action.list.basic.AbstractBasicListAction;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.paginator.basic.ListIFace;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericList;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.workflow.api.IWorkflowAssignment;
import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowEngine;
import it.eng.spago.workflow.api.WorkflowEngineException;

import java.util.Iterator;
import java.util.List;

/**
 * @author Zoppello
 *
 * A Spago List Action  that use SpagoWorkflowAPI to get the worklist ( the list of assignments for 
 * which the user is a candidate ) for the user in the given session
 */
public class WorkListAction extends AbstractBasicListAction{

	
	
	/**
	 * Get the list bof the assignments for the user in current session and give the list of the assignment
	 * as a sourceBean that will be used by a Spago List Tag
	 * @return The list of assigment as a SourceBean 
	 * @throws Exception if some errors occurs
	 */
	private SourceBean getAssigments() throws Exception {
		    	
    	//Getting Containers
    	SessionContainer session = getRequestContainer().getSessionContainer();
    	ApplicationContainer application = ApplicationContainer.getInstance();
    	IEngUserProfile userProfile = (IEngUserProfile)session.getAttribute(IEngUserProfile.ENG_USER_PROFILE);
    	
    	//Getting Workflow Engine 
    	IWorkflowEngine wfEngine = (IWorkflowEngine)application.getAttribute("WfEngine");
    	
    	
    	
    	//Getting Connection From Workflow
    	IWorkflowConnection wfConnection = wfEngine.getWorkflowConnection();
    	
    	
    	// Open the connection
    	wfConnection.open((String)userProfile.getUserUniqueIdentifier(), (String)userProfile.getUserAttribute("password"));
        
       
        List worklist = wfConnection.getAssignments();
        
        // Transform List to sourceBean
        SourceBean workListAsSourceBean = assignmentsToSourceBean(worklist);
        
        return workListAsSourceBean;
        
	}
	
	
	/**
	 * Map a java.util.List of assignments to a source bean
	 * @param aWorkList - The java.util.List of the assignment
	 * @return The source bean structure of the Assignments List
	 * @throws SourceBeanException if some errors occurs in the source bean population
	 * @throws WorkflowEngineException if some errors occurs
	 */
	protected SourceBean assignmentsToSourceBean (List aWorkList) throws SourceBeanException, WorkflowEngineException{
		SourceBean rows = new SourceBean("ROWS");
        
		for (Iterator it = aWorkList.iterator(); it.hasNext();){
        	rows.setAttribute(adapt((IWorkflowAssignment)it.next()));
        }
        
        return rows; 
	}
	
	

	/** Map a single assignment to a source bean
	 * @param wfAssignment The assignment to map to the source bean
	 * @return the source bean form of the assignment
	 * @throws SourceBeanException if some errors occurs in the source bean population
	 * @throws WorkflowEngineException if some errors occurs
	 */
	protected SourceBean adapt(IWorkflowAssignment wfAssignment) throws SourceBeanException, WorkflowEngineException {
		SourceBean row = new SourceBean(DataRow.ROW_TAG);
		row.setAttribute("ActivityDescription",(Object)wfAssignment.getActivityDescription());
		row.setAttribute("ActivityKey",wfAssignment.getActivityKey());
		row.setAttribute("ActivityName",wfAssignment.getActivityName());
		row.setAttribute("ActivityPriority",new Integer(wfAssignment.getActivityPriority()));
		row.setAttribute("ActivityState",wfAssignment.getActivityState());
		row.setAttribute("ProcessDescription",wfAssignment.getProcessDescription());
		row.setAttribute("ProcessKey",wfAssignment.getProcessKey());
		row.setAttribute("ProcessName",wfAssignment.getProcessName());
		row.setAttribute("ProcessState",wfAssignment.getProcessState());
		row.setAttribute("Accepted",new Boolean(wfAssignment.isAccepted()));
		row.setAttribute("TargetForm",wfAssignment.getMappedForm());
		return row;
		
	}
	
    

	
	
	
	/**
	 * Ovverided to code the logic to get the worklist from workflow engine 
	 */
	public ListIFace getList(SourceBean arg0, SourceBean arg1) throws Exception {
		PaginatorIFace paginator = new GenericPaginator();
		InitializerIFace serviceInitializer = (InitializerIFace) this;
		
		int pagedRows = 10;
		
		
		paginator.setPageSize(pagedRows);
		
		RequestContextIFace serviceRequestContext = (RequestContextIFace) this;
		
		// Chiamata al workflow
		SourceBean rowsSourceBean = getAssigments();
		
		
		List rowsVector = null;
		if (rowsSourceBean != null)
			rowsVector = rowsSourceBean.getAttributeAsList(DataRow.ROW_TAG);
		if ((rowsSourceBean == null)) {//|| (rowsVector.size() == 0)) {
			EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.INFORMATION, 10001));
		} // if ((rowsSourceBean == null) || (rowsVector.size() == 0))
		else{
			for (int i = 0; i < rowsVector.size(); i++)
				paginator.addRow(rowsVector.get(i));
		}
		ListIFace list = new GenericList();
		list.setPaginator(paginator);
		// list.addStaticData(firstData);
		return list;
	} // public static ListIFace getList(ServiceIFace service, SourceBean)
	

}
