
package it.eng.spago.workflow.worklist.initializer;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.workflow.api.IWorkflowEngine;
import it.eng.spago.workflow.api.IWorkflowEngineFactory;

import java.util.Iterator;
import java.util.List;


/**
 * @author zoppello
 *
 * A Spago Initializer that read the file workflow.xml with the following structure:
 * 
 * <code> 
 * <?xml version="1.0" encoding="ISO-8859-1"?>
 * <WORKFLOW>
 *		<ENGINE name="shark01" factory="it.eng.spago.workflow.shark.impl.SharkWorkflowEngineFactoryImpl">
 *			<LOAD-PACKAGE packageFile="SanitaBO.xpdl"/>
 *	    </ENGINE>
 * </WORKFLOW>
 * </code>
 * The Initializer istantiate the engine, load ad startup the paclages listed in <LOAD-PACKAGE> sub elements and
 * put the engine instance on Spago Application Container with the WfEngine Key
 * 
 * So to get the instance of the engine we must code something like:
 * <code> IWorkflowEngine wfEngine = (IWorkflowEngine)ApplicationContainer.getInstance().setAttribute("WfEngine")  </code>
 */
public class WfInitializer implements InitializerIFace {
    

    /**
     * Default Constructor
     */
    public WfInitializer() {
        super();
       
    } // public WfInitializer()

    
    /**
     * Initialize the Workflow Engine reading configuration from workflow.xml file
     */
    public void init(SourceBean config) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "WfInitializer::init: config", config);
        
        try {
        	
        	SourceBean wfEngineDescriptor =  (SourceBean)ConfigSingleton.getInstance().getAttribute("WORKFLOW.ENGINE");
            
        	String engineName = null;
            String engineFactoryClass = null;
            IWorkflowEngineFactory wfEngineFactory = null;
            IWorkflowEngine wfEngine = null;
            List loadPackageList = null;
            
            SourceBean loadPackageSourceBean =  null;
            String packageFileName = null;
            
            engineName = (String)wfEngineDescriptor.getAttribute("name");
            engineFactoryClass = (String)wfEngineDescriptor.getAttribute("factory");
            
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "WfInitializer::Try to initialize engine: ["+engineName+"] ["+engineFactoryClass+"]");
            	
            wfEngineFactory = (IWorkflowEngineFactory)Class.forName(engineFactoryClass).newInstance();
            wfEngine = wfEngineFactory.getWorkflowEngine();
            	 
            loadPackageList =  wfEngineDescriptor.getAttributeAsList("LOAD-PACKAGE");
            for (Iterator itLoadPackage = loadPackageList.iterator(); itLoadPackage.hasNext(); ){
            		 loadPackageSourceBean = (SourceBean)itLoadPackage.next();
            		 packageFileName = (String)loadPackageSourceBean.getAttribute("packageFile");
            		 wfEngine.loadPackageDefinition(packageFileName);
            }
            	
            ApplicationContainer.getInstance().setAttribute("WfEngine",wfEngine);
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "WfInitializer:: OK Initialized Engine: ["+engineName+"] ["+engineFactoryClass+"]");
            
        } // try
        catch (Throwable t) {
        	t.printStackTrace();
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "WfInitializer::init:", (Exception)t);
        } // catch (EMFInternalError eie)
    } // public void init(SourceBean config)

	/**
	 * @see it.eng.spago.init.InitializerIFace#getConfig()
	 */
	public SourceBean getConfig() {
		// TODO Auto-generated method stub
		return null;
	}

    
} // public class WfInitializer implements InitializerIFace
