package it.eng.spago.workflow.worklist.modules;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dbaccess.sql.DataRow;
import it.eng.spago.dispatching.module.list.basic.AbstractBasicListModule;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.paginator.basic.ListIFace;
import it.eng.spago.paginator.basic.PaginatorIFace;
import it.eng.spago.paginator.basic.impl.GenericList;
import it.eng.spago.paginator.basic.impl.GenericPaginator;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.workflow.api.IWorkflowConnection;
import it.eng.spago.workflow.api.IWorkflowEngine;
import it.eng.spago.workflow.api.IWorkflowPackage;
import it.eng.spago.workflow.api.IWorkflowProcess;
import it.eng.spago.workflow.api.WorkflowEngineException;

import java.util.Iterator;
import java.util.List;
/**
 * @author zoppello
 *
 * A Spago List module that retrieve the process ( run time instances ) for a given package
 */
public class ProcessForPackageModule extends AbstractBasicListModule {

	/**
	 * This method use the Shark Workflow API to get the list of the run time process for the package id retrived in
	 * service request
	 * @return The source bean representing the list of the process for the package
	 * @throws Exception if some errors occurs
	 */
	private SourceBean getProcessForPackage() throws Exception {
		//		Getting Containers
    	SessionContainer session = getRequestContainer().getSessionContainer();
    	ApplicationContainer application = ApplicationContainer.getInstance();
    	
    	IEngUserProfile userProfile = (IEngUserProfile)session.getAttribute(IEngUserProfile.ENG_USER_PROFILE);
    	
    	//Getting Workflow Engine 
    	IWorkflowEngine wfEngine = (IWorkflowEngine)application.getAttribute("WfEngine");
    	
    	
    	
    	//Getting Connection From Workflow
    	IWorkflowConnection wfConnection = wfEngine.getWorkflowConnection();
    	
    	
    	// Open the connection
    	wfConnection.open((String)userProfile.getUserUniqueIdentifier(), (String)userProfile.getUserAttribute("password"));
        
       
        String packageID = (String)getRequestContainer().getServiceRequest().getAttribute("PackageID");
        
        IWorkflowPackage pkg = wfEngine.getWorkflowPackage(packageID);
        
        List processListForPackage = wfConnection.getProcessForPackage(pkg);
        
        SourceBean processListForPackageAsSourceBean = processListToSourceBean(processListForPackage, wfConnection);
        
        return processListForPackageAsSourceBean;
	}
	
	/**
	 * Transform a Process List in Source Bean form 
	 * @param aProcessList The java.util.List of IWorkflowProcess Object
	 * @return the source bean form of the process list
	 * @throws SourceBeanException if some errors occurs during source bean population
	 * @throws WorkflowEngineException if some errors occurs 
	 */
	protected SourceBean processListToSourceBean (List aProcessList, IWorkflowConnection wfConnection) throws SourceBeanException, WorkflowEngineException{
		SourceBean rows = new SourceBean("ROWS");
        
		for (Iterator it = aProcessList.iterator(); it.hasNext();){
        	rows.setAttribute(adapt((IWorkflowProcess)it.next()));
        }
        
        return rows; 
	}
	
	/**
	 * Transform a simple IWorkflowProcess Object in source bean form
	 * @param wfProcess - The object to be transformed in source bean
	 * @return The source bean form of IWorkflowProcess
	 * @throws SourceBeanException if some errors occurs during source bean population
	 * @throws WorkflowEngineException if some errors occurs
	 */
	protected SourceBean adapt(IWorkflowProcess wfProcess) throws SourceBeanException, WorkflowEngineException {
		SourceBean row = new SourceBean(DataRow.ROW_TAG);
		
		row.setAttribute("ProcessID",wfProcess.getId());
		row.setAttribute("ProcessName",wfProcess.getName());
		row.setAttribute("ProcessDescription",wfProcess.getDescription());
		row.setAttribute("ProcessState",wfProcess.getState());
		
		
		
		return row;
		
	}
	/** 
	 * Ovverride the getList method to get The process list for a given package id from Workflow Engine
	 */
	public ListIFace getList(SourceBean arg0, SourceBean arg1) throws Exception {
		PaginatorIFace paginator = new GenericPaginator();
		InitializerIFace serviceInitializer = (InitializerIFace) this;
		
		int pagedRows = 10;
		
		
		paginator.setPageSize(pagedRows);
		
		RequestContextIFace serviceRequestContext = (RequestContextIFace) this;
		
		// Chiamata al workflow
		SourceBean rowsSourceBean = getProcessForPackage();
		
		
		List rowsVector = null;
		if (rowsSourceBean != null)
			rowsVector = rowsSourceBean.getAttributeAsList(DataRow.ROW_TAG);
		if ((rowsSourceBean == null)) {//|| (rowsVector.size() == 0)) {
			EMFErrorHandler engErrorHandler = serviceRequestContext.getErrorHandler();
			engErrorHandler.addError(new EMFUserError(EMFErrorSeverity.INFORMATION, 10001));
		} // if ((rowsSourceBean == null) || (rowsVector.size() == 0))
		else{
			for (int i = 0; i < rowsVector.size(); i++)
				paginator.addRow(rowsVector.get(i));
		}
		ListIFace list = new GenericList();
		list.setPaginator(paginator);
		// list.addStaticData(firstData);
		return list;
	} // public static ListIFace getList(ServiceIFace service, SourceBean)
	
}
