
package it.eng.spago.workflow.worklist.publisher;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.presentation.PublisherDispatcherIFace;

/**
 * @author Zoppello
 *
 * A Spago Java Publisher that contain logic to generate a custom or a self described form if
 * it find in response the attribute Target Form
 * 
 * This publisher is for AcceptActivity Action that provide to put in service response the attribute target form
 * if it's necessary
 */
public class AcceptActivityPublisher implements PublisherDispatcherIFace {
	
	public AcceptActivityPublisher() {
		super();

	}

	public String getPublisherName(RequestContainer request,
			ResponseContainer response) {

		String mappedForm = (String)response.getServiceResponse().getAttribute("TargetForm");
	
		if (mappedForm != null) {
			// The activity is mapped to a form
			// Let's retrive the form descriptor
			SourceBean formDescriptor = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute(
                    "FORMS.FORM",
                    "formIdentifier",
                    mappedForm);
			if (formDescriptor != null){
				// The method determine publisher has the logic to determine publisher
				return determinePublisher(formDescriptor);
			}else{
				// Trace and return to WORKLIST_PUBLISHER
				return "LOOP_WORKLIST_PUBLISHER";
			}// end else
		}else{
			//Trace and return to WORKLIST_PUBLISHER
			return "LOOP_WORKLIST_PUBLISHER";
		}//end else
	}
	
	public String determinePublisher(SourceBean aFormDescriptor){
		// 1) AUTO-GENERATED 
		//    In this case the forms are autogenerated by spago by the rules defined in the form tags
		//    In that case spago do all the monkey job
		//	  <FORM type="AUTO"/>
		// 2) CUSTOM FORMS
		// 	  We can have forms defined as follow where the form is in a custom page
		// 	  In this case 
		// 	  <FORM id="formId" type="CUSTOM" formCustomPublisher="aSpagoPublisher" />
		//    !!!!!!!!!!!!!!!!!!!!!!! ATTENTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//    In that case the jsp writer is responsible to put a field called ActivityKey on page
		//
		String formType = (String)aFormDescriptor.getAttribute("formType");
		if (formType.equalsIgnoreCase("CUSTOM")){
			return (String)aFormDescriptor.getAttribute("formCustomPublisher");
		}else{
			return "AUTO_GENERATE_FORM_PUBLISHER";
		}
	}
}

