

package it.eng.spago.workflow.worklist.tags;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.RequestContainerAccess;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * @author zoppello
 * 
 * Tags to generate a HTML form described in file forms.xml 
 * 
 * This tags generate HMTL code conforming to Italian laws about Web Accessibility Guidelines
 * 
 */

public class FormTag extends TagSupport
{
	
    
    private HttpServletRequest httpRequest = null;
    private HttpServletResponse httpResponse = null;
    private RequestContainer requestContainer = null;
    private ResponseContainer responseContainer = null;
    private String contextPath = null;
    
    private SourceBean formDescriptor = null;
    
    private String formIdentifier = null;
    private String method = null;
    private String action = null;
    private String layout = "plain";
    private StringBuffer htmlStream = null;
    
    /**
     *Consructor
     */
    public FormTag()
    {
    		
    }

    
    
   
    
    public int doStartTag() throws JspException {
    	
    	this.httpResponse = ((HttpServletResponse)pageContext.getResponse());
    	this.httpRequest = ((HttpServletRequest)pageContext.getRequest());
    	
    	this.contextPath = httpRequest.getContextPath();
    	
    	this.requestContainer = RequestContainerAccess.getRequestContainer(this.httpRequest);
    	
    	this.responseContainer = ResponseContainerAccess.getResponseContainer(this.httpRequest);
     	
    	TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.INFORMATION,
            "Forms::doStartTag:: invocato");
     	
        
        
        
        ConfigSingleton configure = ConfigSingleton.getInstance();
        this.formDescriptor = 
                (SourceBean) configure.getFilteredSourceBeanAttribute(
                    "FORMS.FORM",
                    "formIdentifier",
                    this.formIdentifier);
        
        htmlStream = new StringBuffer();
        makeForm();
        try {
            pageContext.getOut().print(htmlStream);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DefaultListTag::doStartTag:: Impossibile inviare lo stream");
            throw new JspException("Impossibile inviare lo stream");
        } // catch (Exception ex)
        return SKIP_BODY;
    } // public int doStartTag() throws JspException

    
	
	
	
	
	/**
	 * Creates a form into the jsp page.
	 * 
	 * @throws JspException If any exception occurs.
	 */
	
	protected void makeForm() throws JspException {
		htmlStream.append("<form id='"+this.layout+"Form' action='"+this.action+"' method='"+this.method+"'>\n");
		List fieldSets = formDescriptor.getAttributeAsList("FIELDSET");
		SourceBean fieldSet = null;
		String fieldSetTypeStr = null;
		for (Iterator it = fieldSets.iterator(); it.hasNext(); ){
			fieldSet = (SourceBean)it.next();
			fieldSetTypeStr = (String)fieldSet.getAttribute("type");
			if (fieldSetTypeStr != null && fieldSetTypeStr.equalsIgnoreCase("radio")){
				makeRadioField(fieldSet);
			}else{
				makeFieldSet(fieldSet);
			}
		}
		makeSubmitButtons();
		htmlStream.append("</form>\n");
		
	} // public void makeForm()
	
	/**
	 * Produce the html code that generate the submits button
	 */
	protected void makeSubmitButtons(){
		List submits = formDescriptor.getAttributeAsList("ONSUBMIT");
		SourceBean submitSb = null;
		String submitName = null;
		String submitValue = null;
		
		for (Iterator it = submits.iterator(); it.hasNext(); ){
			submitSb = (SourceBean)it.next();
			submitName = (String)submitSb.getAttribute("name");
			submitValue = (String)submitSb.getAttribute("value");
			htmlStream.append("<input type='submit' name='"+submitName+"' value='"+submitValue+"'/>");
		}
	}
	
	/**
	 * Produce the html code that generate the field set
	 */
	protected void makeFieldSet( SourceBean fieldSetSourceBean) throws JspException {
		htmlStream.append("<fieldset>\n");
		htmlStream.append("<legend>\n");
		String legendStr = (String)fieldSetSourceBean.getAttribute("legend");
		if (legendStr != null)
			htmlStream.append("		"+legendStr+"\n");
		htmlStream.append("</legend>\n");
		List fields = fieldSetSourceBean.getAttributeAsList("FIELD");
		int indexInFieldSet = 0;
		for (Iterator it = fields.iterator(); it.hasNext(); indexInFieldSet++){
			makeField((SourceBean)it.next(), indexInFieldSet);
		}
				
		htmlStream.append("</fieldset>\n");
	} // public void makeForm()
	
	/**
	 * Produce the html code that generate a single field
	 */
	protected void makeField(SourceBean field, int indexInFieldSet) throws JspException {
		String fieldType = (String)field.getAttribute("type");
		if (fieldType.equalsIgnoreCase("hidden")){
			makeHiddenField(field, indexInFieldSet);
		}
		if (fieldType.equalsIgnoreCase("text")){
			makeTextField(field, indexInFieldSet);
		}if (fieldType.equalsIgnoreCase("radio")){
			makeRadioField(field);
		}if (fieldType.equalsIgnoreCase("select")){
			makeSelectField(field, indexInFieldSet);
		}if (fieldType.equalsIgnoreCase("checkbox")){
			makeCheckBoxField(field, indexInFieldSet);
		}else{
			return;
		}
	} // public void makeForm()
	
	/**
	 * Produce the html code that generate a text field
	 */
	protected void makeTextField(SourceBean field, int indexInFieldSet) throws JspException {
		String fieldName = (String)field.getAttribute("name");
		String fieldLabel = (String)field.getAttribute("label");
		String fieldLabelType = (String)field.getAttribute("labelType");
		if (indexInFieldSet == 0){
			htmlStream.append("<label for='"+fieldName+"' class='first'>\n");
		}else{
			htmlStream.append("<label for='"+fieldName+"'>\n");
		}
		
		if (fieldLabelType.equalsIgnoreCase("pre")){
			htmlStream.append(fieldLabel+"\n");
		}
		htmlStream.append("<input id='"+fieldName+"' name='"+fieldName+"' type='text' value=''/>");
		if (fieldLabelType.equalsIgnoreCase("post")){
					htmlStream.append(fieldLabel+"\n");
		}
		htmlStream.append("</label>\n");
	}
	
	/**
	 * Produce the html code that generate a hidden field
	 */
	protected void makeHiddenField(SourceBean field, int indexInFieldSet) throws JspException {
		String fieldName = (String)field.getAttribute("name");
		String fieldValueExpr = (String)field.getAttribute("value");
		
		String fieldValue = getValueFromExpression(fieldValueExpr);
		htmlStream.append("<input id='"+fieldName+"' name='"+fieldName+"' type='hidden' value='"+fieldValue+"'/>");		
		htmlStream.append("</label>\n");
	}
	
	private String getValueFromExpression(String paramValueExpr){
		if ((paramValueExpr == null) || (paramValueExpr.trim().length() == 0))
			return paramValueExpr;
		if (!paramValueExpr.startsWith("$")){
			return paramValueExpr;
		}else{ 
			String scope = paramValueExpr.substring(0, paramValueExpr.indexOf("."));
			String fieldName = paramValueExpr.substring(paramValueExpr.indexOf(".")+1);
			System.out.println("Getting field name ["+fieldName+"]  From ["+ scope +"]");
			if (paramValueExpr.startsWith("$REQUEST_CONTAINER")){
				return (String)this.requestContainer.getAttribute(fieldName);
			}else if (paramValueExpr.startsWith("$RESPONSE_CONTAINER")){
				return (String)this.responseContainer.getAttribute(fieldName);
			}else if (paramValueExpr.startsWith("$SERVICE_REQUEST")){
				return (String)this.requestContainer.getServiceRequest().getAttribute(fieldName);
			}else if (paramValueExpr.startsWith("$SERVICE_RESPONSE")){
				return (String)this.responseContainer.getServiceResponse().getAttribute(fieldName);
			}else if (paramValueExpr.startsWith("$SESSION_CONTAINER")){
				return (String)this.requestContainer.getSessionContainer().getAttribute(fieldName);
			}else{
				return (String)this.requestContainer.getServiceRequest().getAttribute(fieldName);
			}
		}
	}
	
	/**
	 * Produce the html code that generate a radio field
	 */
	protected void makeRadioField(SourceBean field) throws JspException {
		
		String fieldName = (String)field.getAttribute("name");
		String fieldLabel = (String)field.getAttribute("label");
		String fieldLabelType = (String)field.getAttribute("labelType");
		htmlStream.append("<fieldset class='radio'>\n");
		htmlStream.append("<legend>\n");
		htmlStream.append("		"+fieldLabel+"\n");
		htmlStream.append("</legend>\n");
		List options = field.getAttributeAsList("FIELD-OPTION");
		SourceBean optionSourceBean = null;
		
		String radioId = null;
//		String optionLabel = null;
		String optionValue = null;
		int i=0;
		for (Iterator it = options.iterator(); it.hasNext(); i++){
			optionSourceBean = (SourceBean)it.next();
			radioId = fieldName+"_Option_"+i;
//			optionLabel = (String)optionSourceBean.getAttribute("optionLabel");
			optionValue = (String)optionSourceBean.getAttribute("optionValue");
			htmlStream.append("<label for='"+radioId+"'>\n");
			htmlStream.append("<input id='"+radioId+"' name='"+fieldName+"' type='radio' value='"+optionValue+"'/>\n");
			htmlStream.append(optionSourceBean.getAttribute("optionLabel")+"\n");
			htmlStream.append("</label>\n");
		}
		htmlStream.append("</fieldset>");
	}
	
	/**
	 * Produce the html code that generate a select field
	 */
	protected void makeSelectField(SourceBean field, int indexInFieldSet) throws JspException {
		
		String fieldName = (String)field.getAttribute("name");
		String fieldLabel = (String)field.getAttribute("label");
		String fieldLabelType = (String)field.getAttribute("labelType");
		if (indexInFieldSet == 0){
			htmlStream.append("<label for='"+fieldName+"' class='first'>\n");
		}else{
			htmlStream.append("<label for='"+fieldName+"'>\n");
		}
		if (fieldLabelType.equalsIgnoreCase("pre")){
			htmlStream.append(fieldLabel+"\n");
		}
		
		htmlStream.append("<select id='"+fieldName+"' name='"+fieldName+"'/>");
		List options = field.getAttributeAsList("FIELD-OPTION");
		SourceBean optionSourceBean = null;
		
		
		String optionLabel = null;
		String optionValue = null;
		int i=0;
		for (Iterator it = options.iterator(); it.hasNext(); i++){
			optionSourceBean = (SourceBean)it.next();
			optionLabel = (String)optionSourceBean.getAttribute("optionLabel");
			optionValue = (String)optionSourceBean.getAttribute("optionValue");
			htmlStream.append("<option value='"+optionValue+"'>"+optionLabel+"</option>\n");
		}
		htmlStream.append("</select>\n");
		if (fieldLabelType.equalsIgnoreCase("post")){
					htmlStream.append(fieldLabel+"\n");
		}
		htmlStream.append("</label>\n");
	}
	
	/**
	 * Produce the html code that generate a checkbox field
	 */
	protected void makeCheckBoxField(SourceBean field, int indexInFieldSet) throws JspException {
		String fieldName = (String)field.getAttribute("name");
		String fieldLabel = (String)field.getAttribute("label");
		String fieldLabelType = (String)field.getAttribute("labelType");
		return;
	}
	
	/**
	 * @see javax.servlet.jsp.tagext.Tag#doEndTag()
	 */
	public int doEndTag() throws JspException {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "ListTag::doEndTag:: invocato");
		
		return super.doEndTag();
		
	} // public int doEndTag() throws JspException





	public String getAction() {
		return action;
	}





	public void setAction(String action) {
		this.action = action;
	}





	public String getContextPath() {
		return contextPath;
	}





	public void setContextPath(String contextPath) {
		this.contextPath = contextPath;
	}





	public SourceBean getFormDescriptor() {
		return formDescriptor;
	}





	public void setFormDescriptor(SourceBean formDescriptor) {
		this.formDescriptor = formDescriptor;
	}





	public String getFormIdentifier() {
		return formIdentifier;
	}





	public void setFormIdentifier(String formIdentifier) {
		this.formIdentifier = formIdentifier;
	}





	public StringBuffer getHtmlStream() {
		return htmlStream;
	}





	public void setHtmlStream(StringBuffer htmlStream) {
		this.htmlStream = htmlStream;
	}





	public HttpServletRequest getHttpRequest() {
		return httpRequest;
	}





	public void setHttpRequest(HttpServletRequest httpRequest) {
		this.httpRequest = httpRequest;
	}





	public HttpServletResponse getHttpResponse() {
		return httpResponse;
	}





	public void setHttpResponse(HttpServletResponse httpResponse) {
		this.httpResponse = httpResponse;
	}





	public String getLayout() {
		return layout;
	}





	public void setLayout(String layout) {
		this.layout = layout;
	}





	public String getMethod() {
		return method;
	}





	public void setMethod(String method) {
		this.method = method;
	}
} // public class ListTag extends TagSupport



































